-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

local RC = RaidChecklist

if not RC then return end

-- Map from GUID to name
RC.provider_names = {}

-- Map from (de)buff key to list of GUIDs currently providing said de(buff)
RC.providers = {}
for i,b in ipairs (RC.buffs) do RC.providers[b] = {} end
for i,b in ipairs (RC.debuffs) do RC.providers[b] = {} end
for i,b in ipairs (RC.dispels) do RC.providers[b] = {} end
for i,b in ipairs (RC.ccpvps) do RC.providers[b] = {} end

local function remove_provider (rguid)
  -- Iterate over all the (de)buffs
  for key,guidlist in pairs (RC.providers) do
    guidlist[rguid] = nil
  end
  if (RC.OnRefresh) then RC:OnRefresh () end
end

local function add_provider (buff, guid)
  RC.providers[buff][guid] = true
  if (RC.OnRefresh) then RC:OnRefresh () end
end

-- Works out which spec is used based on talent point allocations.
-- Returns 0 if no main tree is spec'd into, else 1-3 indicating tree.
local function which_spec (n1, n2, n3)
  return (n1 > n2 and n1 > n3 and 1) or (n2 > n1 and n2 > n3 and 2) or (n3 > n1 and n3 > n2 and 3) or 0
end

-- Matrix based on select(2, UnitBaseClass(unit)) and spec (0-4) to a list of (de)buffs provided
-- This is almost a duplication of the "availablefrom" table over in raid_checklist, but that one
-- has to be human readable and there was just no neat way to satisfy both in a reasonable manner :(
local matrix = {
  WARRIOR = {
    [0] = { "AGISTR", "STAM", "ATTACKSPD", "SILENCE", "SNARE", "DISARM", "STUN" }, -- All warriors can provide these even without a spec
    [1] = { "AGISTR", "STAM", "ATTACKSPD", "SILENCE", "SNARE", "DISARM", "STUN", "PHYSDMGTAKEN", "BLEEDDMGTAKEN", "MORTALSTRIKE" }, -- Arms
    [2] = { "AGISTR", "STAM", "ATTACKSPD", "SILENCE", "SNARE", "DISARM", "STUN", "CRIT", "MORTALSTRIKE" }, -- Fury
    [3] = { "AGiSTR", "STAM", "ATTACKSPD", "SILENCE", "SNARE", "DISARM", "STUN", "DEMOROAR", "SUNDER" }, -- Protection
  },
  HUNTER = {
    [0] = { "MORTALSTRIKE", "DISORIENT", "SNARE", "ENRAGE" },
    [1] = { "MORTALSTRIKE", "DISORIENT", "SNARE", "ENRAGE", "ALLDMG", "STUN" }, -- BM
    [2] = { "MORTALSTRIKE", "DISORIENT", "SNARE", "ENRAGE", "ATTACKPWR", "SILENCE" }, -- Marksman
    [3] = { "MORTALSTRIKE", "DISORIENT", "SNARE", "ENRAGE", "PHYSHASTE", "ROOT" }, -- Survival (root = entrapment)
  },
  DEATHKNIGHT = {
    [0] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "SILENCE", "SNARE", "CASTSPD" },
    [1] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "SILENCE", "SNARE", "CASTSPD", "DEMOROAR" }, -- Blood
    [2] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "SILENCE", "SNARE", "CASTSPD", "PHYSHASTE", "PHYSDMGTAKEN", "STUN" }, -- Frost
    [3] = { "AGISTR", "ATTACKPWR", "ATTACKSPD", "SILENCE", "SNARE", "CASTSPD", "SPELLDMGTAKEN" }, -- Unholy
  },
  PRIEST = {
    [0] = { "STAM", "BURSTMANA", "DEFMAGIC", "OFFMAGIC" },
    [1] = { "STAM", "BURSTMANA", "DEFMAGIC", "OFFMAGIC", "DISEASE" }, -- Discipline
    [2] = { "STAM", "BURSTMANA", "DEFMAGIC", "OFFMAGIC", "DISEASE" }, -- Holy
    [3] = { "STAM", "BURSTMANA", "DEFMAGIC", "OFFMAGIC", "SPELLHASTE", "REPLENISHMENT", "MORTALSTRIKE", "SILENCE", "SNARE", "DISARM" }, -- Shadow
  },
  SHAMAN = {
    [0] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN", "SILENCE", "SNARE", "ARMOR", "PUSHBACKRES", "ATTACKSPD", "OFFMAGIC", "CURSE" },
    [1] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN", "SILENCE", "SNARE", "ARMOR", "PUSHBACKRES", "ATTACKSPD", "OFFMAGIC", "CURSE", "CRIT", "SPELLPWR10", "ROOT", "KNOCKBACK" }, -- Elemental
    [2] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN", "SILENCE", "SNARE", "ARMOR", "PUSHBACKRES", "ATTACKSPD", "OFFMAGIC", "CURSE", "ATTACKPWR" }, -- Enhancement
    [3] = { "AGISTR", "SPELLPWR6", "PHYSHASTE", "SPELLHASTE", "BLOODLUST", "MANAREGEN", "SILENCE", "SNARE", "ARMOR", "PUSHBACKRES", "ATTACKSPD", "OFFMAGIC", "CURSE", "BURSTMANA", "DEFMAGIC" }, -- Restoration
  },
  PALADIN = {
    [0] = { "STATS", "ATTACKPWR", "MANAREGEN", "STUN", "ARMOR", "PUSHBACKRES", "DISEASE", "POISON" },
    [1] = { "STATS", "ATTACKPWR", "MANAREGEN", "STUN", "ARMOR", "PUSHBACKRES", "DISEASE", "POISON", "DEFMAGIC" }, -- Holy
    [2] = { "STATS", "ATTACKPWR", "MANAREGEN", "STUN", "ARMOR", "PUSHBACKRES", "DISEASE", "POISON", "ATTACKSPD", "DEMOROAR", "SILENCE" }, -- Protection
    [3] = { "STATS", "ATTACKPWR", "MANAREGEN", "STUN", "ARMOR", "PUSHBACKRES", "DISEASE", "POISON", "REPLENISHMENT", "ALLDMG" }, -- Retribution
  },
  DRUID = {
    [0] = { "STATS", "ROOT", "BURSTMANA", "POISON", "CURSE", "ENRAGE" },
    [1] = { "STATS", "ROOT", "BURSTMANA", "POISON", "CURSE", "ENRAGE", "SPELLHASTE", "SPELLDMGTAKEN", "SILENCE", "KNOCKBACK" }, -- Balance
    [2] = { "STATS", "ROOT", "BURSTMANA", "POISON", "CURSE", "ENRAGE", "CRIT", "BLEEDDMGTAKEN", "ATTACKSPD", "DEMOROAR", "SUNDER", "SILENCE", "SNARE", "STUN" }, -- Feral (demo/sunder only in bear though)
    [3] = { "STATS", "ROOT", "BURSTMANA", "POISON", "CURSE", "ENRAGE", "REPLENISHMENT", "DEFMAGIC" }, -- Restoration
  },
  WARLOCK = {
    [0] = { "SPELLDMGTAKEN", "CASTSPD", "DISORIENT", "SNARE", "DEMOROAR" },
    [1] = { "SPELLDMGTAKEN", "CASTSPD", "DISORIENT", "SNARE", "DEMOROAR" }, -- Affliction
    [2] = { "SPELLDMGTAKEN", "CASTSPD", "DISORIENT", "SNARE", "DEMOROAR", "SPELLPWR10"  }, -- Demonology
    [3] = { "SPELLDMGTAKEN", "CASTSPD", "DISORIENT", "SNARE", "DEMOROAR", "REPLENISHMENT", "SPELLCRIT" }, -- Destruction (spell crit accessible from other trees though "Shadow and Flame")
  },
  ROGUE = {
    [0] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "DISORIENT", "SILENCE", "SNARE", "DISARM", "ENRAGE" },
    [1] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "DISORIENT", "SILENCE", "SNARE", "DISARM", "ENRAGE", "SPELLDMGTAKEN" }, -- Assasination
    [2] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "DISORIENT", "SILENCE", "SNARE", "DISARM", "ENRAGE", "PHYSDMGTAKEN" }, -- Combat
    [3] = { "CASTSPD", "MORTALSTRIKE", "SUNDER", "DISORIENT", "SILENCE", "SNARE", "DISARM", "ENRAGE", "CRIT", "BLEEDDMGTAKEN" }, -- Subtlety
  },
  MAGE = {
    [0] = { "SPELLPWR6", "BLOODLUST", "MANA", "SILENCE", "ROOT" },
    [1] = { "SPELLPWR6", "BLOODLUST", "MANA", "SILENCE", "ROOT", "CASTSPD", "ALLDMG", "SNARE" }, -- Arcane
    [2] = { "SPELLPWR6", "BLOODLUST", "MANA", "SILENCE", "ROOT", "SPELLCRIT", "DISORIENT" }, -- Fire
    [3] = { "SPELLPWR6", "BLOODLUST", "MANA", "SILENCE", "ROOT", "REPLENISHMENT", "STUN" }, -- Frost
  }
}

local function provides (class, spec)
  return matrix[class][spec]
end

RC.OnTalentUpdate = function (self, event, guid, unit, spec, n1, n2, n3)
  -- Update the talent status
  RC.missing_talents_for = RC.gt:GetTalentMissingNames ()

--if event == "LibGroupTalents_Update" then
  --DEFAULT_CHAT_FRAME:AddMessage ("RC: updated talents for GUID "..(guid or "nil")..", unit "..unit) end

  -- Clear any previous entries for this guid
  remove_provider (guid)

  local spec = which_spec (n1, n2, n3)
  local class = select (2, UnitClassBase (unit))
  local list = matrix[class][spec]
  for i,key in ipairs (list) do
    add_provider (key, guid)
  end
end

RC.OnTalentAdd = function (self, event, guid, unit, name, realm)
  local fullname = (realm and name.."-"..realm) or name
  RC.provider_names[guid] = ClassColorString (select (2,UnitClassBase (unit)))..fullname.."|r"

  -- Update based on class alone to start off with. Better than waiting until in inspect range!
  RC:OnTalentUpdate ("fake", guid, unit, 0, 0, 0, 0)

  --RaidChecklistButton:UNIT_PET(unit) -- We are probably better off waiting for a proper UNIT_PET event I think

  --DEFAULT_CHAT_FRAME:AddMessage ("RC: discovered provider GUID "..(guid or "nil")..", named "..fullname)
end

RC.OnTalentRemove = function (self, event, guid, name, realm)
  -- Update the talent status
  RC.missing_talents_for = RC.gt:GetTalentMissingNames ()

  remove_provider (guid)
  RC.provider_names[guid] = nil

  --DEFAULT_CHAT_FRAME:AddMessage ("RC: removed provider GUID "..(guid or "nil"))
end

RC.gt = LibStub ("LibGroupTalents-1.0")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Update", "OnTalentUpdate")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Add", "OnTalentAdd")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Remove", "OnTalentRemove")

